﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Monnit;
using Monnit.Mine;

namespace SampleMineApplication
{
    public partial class MineSensor2 : Form
    {
        private Sensor _Sensor;

        public MineSensor2(Sensor sensor)
        {
            InitializeComponent();

            _Sensor = sensor;
        }

        private void MineSensor_Load(object sender, EventArgs e)
        {
            if(_Sensor == null)
            {
                MessageBox.Show("Sensor is null");
                this.Close();
            }
            txtSensorID.Text = _Sensor.SensorID.ToString();
            txtSensorType.Text = ((eSensorApplication)_Sensor.MonnitApplicationID).ToString();
            
            cbSendGeneral1.Checked = _Sensor.GeneralConfig1Dirty;
            txtReportInterval.Text = _Sensor.ReportInterval.ToString();
            txtActiveStateInterval.Text = _Sensor.ActiveStateInterval.ToString();
            txtLinkInterval.Text = _Sensor.TransmitIntervalLink.ToString();

            cbSendGeneral2.Checked = _Sensor.GeneralConfig2Dirty;
            txtRetryCount.Text = _Sensor.RetryCount.ToString();
            txtRecovery.Text = _Sensor.Recovery.ToString();

            if (_ApplicationBase.ProfileType(_Sensor.MonnitApplicationID) == eApplicationProfileType.Interval)
            {
                boxProfile1.Height = 148;
                pnlProfileInterval.Visible = true;
                pnlProfileTrigger.Visible = false;
                cbSendProfile1.Checked = _Sensor.ProfileConfig1Dirty;
                txtAssessments.Text = _Sensor.MeasurementsPerTransmission.ToString();
                txtHysteresis.Text = _Sensor.Hysteresis.ToString();
                txtMinThreshold.Text = _Sensor.MinimumThreshold.ToString();
                txtMaxThreshold.Text = _Sensor.MaximumThreshold.ToString();

                boxProfile2.Visible = true;
                cbSendProfile2.Checked = _Sensor.ProfileConfig2Dirty;
                txtCal1.Text = _Sensor.Calibration1.ToString();
                txtCal2.Text = _Sensor.Calibration2.ToString();
                txtCal3.Text = _Sensor.Calibration3.ToString();
                txtCal4.Text = _Sensor.Calibration4.ToString();
            }
            else
            {
                boxProfile2.Visible = false;
                boxProfile1.Height = 174;
                pnlProfileInterval.Visible = false;
                pnlProfileTrigger.Visible = true;
                cbSendProfile1.Checked = _Sensor.ProfileConfig1Dirty;
                ddlDetectionType.SelectedIndex = _Sensor.EventDetectionType;
                txtDetectionPeriod.Text = _Sensor.EventDetectionPeriod.ToString();
                txtDetectionCount.Text = _Sensor.EventDetectionCount.ToString();
                txtRearmTime.Text = _Sensor.RearmTime.ToString();
                ddlBistable.SelectedIndex = _Sensor.BiStable;
            }

        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            bool GeneralConfig1Dirty = false;
            double ReportInterval = 0;
            double ActiveStateInterval = 0;
            int TransmitIntervalLink = 0;

            bool GeneralConfig2Dirty = false;
            int RetryCount = 0;
            int Recovery = 0;

            bool ProfileConfig1Dirty = false;
            int MeasurementsPerTransmission = 0;
            UInt32 Hysteresis = 0;
            UInt32 MinimumThreshold = 0;
            UInt32 MaximumThreshold = 0;

            UInt16 EventDetectionType = 0;
            UInt16 EventDetectionPeriod = 0;
            UInt16 EventDetectionCount = 0;
            UInt16 RearmTime = 0;
            UInt16 BiStable = 0;

            bool ProfileConfig2Dirty = false;
            UInt32 Calibration1 = 0;
            UInt32 Calibration2 = 0;
            UInt32 Calibration3 = 0;
            UInt32 Calibration4 = 0;
            #region Validation
            try
            {
                GeneralConfig1Dirty = cbSendGeneral1.Checked;
                if (!double.TryParse(txtReportInterval.Text, out ReportInterval))
                    throw new Exception("Invalid Report Interval");
                if (!double.TryParse(txtActiveStateInterval.Text, out ActiveStateInterval))
                    throw new Exception("Invalid Active State Interval");
                if (!Int32.TryParse(txtLinkInterval.Text, out TransmitIntervalLink))
                    throw new Exception("Invalid Interval Link");


                GeneralConfig2Dirty = cbSendGeneral2.Checked;
                if (!Int32.TryParse(txtRetryCount.Text, out RetryCount))
                    throw new Exception("Invalid Retry Count");
                if (!Int32.TryParse(txtRecovery.Text, out Recovery))
                    throw new Exception("Invalid Recovery");

                    
                if (_ApplicationBase.ProfileType(_Sensor.MonnitApplicationID) == eApplicationProfileType.Interval)
                {
                    ProfileConfig1Dirty = cbSendProfile1.Checked;
                    if (!Int32.TryParse(txtAssessments.Text, out MeasurementsPerTransmission))
                        throw new Exception("Invalid Assessments");
                    if (!UInt32.TryParse(txtHysteresis.Text, out Hysteresis))
                        throw new Exception("Invalid Hysteresis");
                    if (!UInt32.TryParse(txtMinThreshold.Text, out MinimumThreshold))
                        throw new Exception("Invalid Minimum Threshold");
                    if (!UInt32.TryParse(txtMaxThreshold.Text, out MaximumThreshold))
                        throw new Exception("Invalid Maximum Threshold");

                    ProfileConfig2Dirty = cbSendProfile2.Checked;
                    if (!UInt32.TryParse(txtCal1.Text, out Calibration1))
                        throw new Exception("Invalid Calibration 1");
                    if (!UInt32.TryParse(txtCal2.Text, out Calibration2))
                        throw new Exception("Invalid Calibration 2");
                    if (!UInt32.TryParse(txtCal3.Text, out Calibration3))
                        throw new Exception("Invalid Calibration 3");
                    if (!UInt32.TryParse(txtCal4.Text, out Calibration4))
                        throw new Exception("Invalid Calibration 4");
                }
                else
                {
                    ProfileConfig1Dirty = cbSendProfile1.Checked;
                    EventDetectionType = (UInt16)ddlDetectionType.SelectedIndex;
                    if (!UInt16.TryParse(txtDetectionPeriod.Text, out EventDetectionPeriod))
                        throw new Exception("Invalid Detection Period");
                    if (!UInt16.TryParse(txtDetectionCount.Text, out EventDetectionCount))
                        throw new Exception("Invalid Detection Count");
                    if (!UInt16.TryParse(txtRearmTime.Text, out RearmTime))
                        throw new Exception("Invalid Rearm Time");
                    BiStable = (UInt16)ddlBistable.SelectedIndex;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return;
            }
            #endregion

            #region SetVariables
            _Sensor.GeneralConfig1Dirty = GeneralConfig1Dirty;
            _Sensor.ReportInterval = ReportInterval;
            _Sensor.ActiveStateInterval = ActiveStateInterval;
            _Sensor.TransmitIntervalLink = TransmitIntervalLink;

            _Sensor.GeneralConfig2Dirty = GeneralConfig2Dirty;
            _Sensor.RetryCount = RetryCount;
            _Sensor.Recovery = Recovery;

            if (_ApplicationBase.ProfileType(_Sensor.MonnitApplicationID) == eApplicationProfileType.Interval)
            {
                _Sensor.ProfileConfig1Dirty = ProfileConfig1Dirty;
                _Sensor.MeasurementsPerTransmission = MeasurementsPerTransmission;
                _Sensor.Hysteresis = Hysteresis;
                _Sensor.MinimumThreshold = MinimumThreshold;
                _Sensor.MaximumThreshold = MaximumThreshold;

                _Sensor.ProfileConfig2Dirty = ProfileConfig2Dirty;
                _Sensor.Calibration1 = Calibration1;
                _Sensor.Calibration2 = Calibration2;
                _Sensor.Calibration3 = Calibration3;
                _Sensor.Calibration4 = Calibration4;
            }
            else
            {
                _Sensor.ProfileConfig1Dirty = ProfileConfig1Dirty;
                _Sensor.EventDetectionType = EventDetectionType;
                _Sensor.EventDetectionPeriod = EventDetectionPeriod;
                _Sensor.EventDetectionCount = EventDetectionCount;
                _Sensor.RearmTime = RearmTime;
                _Sensor.BiStable = BiStable;
            }
            #endregion
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
